<?php
// $Id: uc_downpayment.ca.inc,v 1.1 2011/01/17 16:40:41 gjmjr Exp $

/**
 * @file
 * This file contains the Conditional Actions hooks and functions
 */

/**
 * Implementation of hook_ca_entity().
 */
function uc_downpayment_ca_entity() {
  $entities['uc_downpayment'] = array(
    '#title' => t('Ubercart down payment object'),
    '#type' => 'object',
    '#load' => 'uc_downpayment_load',
    '#save' => 'uc_downpayment_save',
  );

  return $entities;
}

/**
 * Implementation of hook_ca_trigger().
 */
function uc_downpayment_ca_trigger() {
  $triggers['uc_downpayment_trigger_payment_due'] = array(
    '#title' => t('Down payment is overdue and set for notification'),
    '#category' => t('Down payment'),
    '#arguments' => array(
      'payment' => array(
        '#entity' => 'uc_downpayment',
        '#title' => t('Down payment'),
      ),
    ),
  );

  return $triggers;
}

/**
 * Implementation of hook_ca_predicate().
 */
function uc_downpayment_ca_predicate() {
  $predicates = array();

  $predicates['uc_downpayment_update_payment'] = array(
    '#title' => t('Update down payment status on order status update'),
    '#description' => t('Update down payment status on order status update.'),
    '#class' => 'downpayment',
    '#status' => 1,
    '#trigger' => 'uc_order_status_update',
    '#conditions' => array(
      '#operator' => 'AND',
      '#conditions' => array(
        array(
          '#name' => 'uc_order_status_condition',
          '#title' => t('Check the order status.'),
          '#argument_map' => array(
            'order' => 'updated_order',
          ),
          '#settings' => array(
            'order_status' => 'completed',
          ),
        ),
      ),
    ),
    '#actions' => array(
      array(
        '#name' => 'uc_downpayment_action_update_status',
        '#title' => t('Update the down payment status'),
        '#argument_map' => array(
          'order' => 'updated_order',
        ),
        '#settings' => array(
            'status' => 1,
         ),
      ),
    ),
  );
  
  $predicates['uc_downpayment_payment_mail'] = array(
    '#title' => t('E-mail customer when down payment due'),
    '#description' => t('E-mail customer when down payment is overdue and set for notification.'),
    '#class' => 'downpayment',
    '#status' => 1,
    '#trigger' => 'uc_downpayment_trigger_payment_due',
    /*
    '#conditions' => array(
      '#operator' => 'AND',
      '#conditions' => array(
        array(
          '#name' => 'uc_downpayment_condition_notified',
          '#title' => t('Check the notification date.'),
          '#argument_map' => array(
            'payment' => 'payment',
          ),
          '#settings' => array(
            'negate' => TRUE,
          ),
        ),
      ),
    ),
    */
    '#actions' => array(
      array(
        '#name' => 'uc_downpayment_action_update_notified',
        '#title' => t('Update notification status'),
        '#argument_map' => array(
          'payment' => 'payment',
        ),
      ),
      array(
        '#name' => 'uc_downpayment_email',
        '#title' => t('Send an down payment email'),
        '#argument_map' => array(
          'payment' => 'payment',
        ),
        '#settings' => array(
          'from' => uc_store_email_from(),
          'addresses' => '[order-email]',
          'subject' => 'Down payment due',
          'message' => "
Hi [order-first-name] [order-last-name],

We wanted to let you know the remaining balance of [downpayment-amount] is due tomorrow.

To guarantee your spot, follow these steps:
1. Go to this link <a href=\"[downpayment-user-url]\">down payment page</a>
2. Login
3. Click on the \"Pay balance\" link next to Down Payment ID [downpayment-id]
4. Finish checkout!

If you have any questions, give us a call at [store-phone].

- [site-name] Team          
          ",
          'format' => 2,
        ),
      ),
    ),
  );

  return $predicates;
}

/**
 * Implementation of hook_ca_condition().
 */
function uc_downpayment_ca_condition() {
  $payment_arg = array(
      '#entity' => 'uc_downpayment',
  );
  
  $conditions['uc_downpayment_condition_notified'] = array(
    '#title' => t('Check if the customer has been notified.'),
    '#category' => t('Down payment'),
    '#description' => t('Returns TRUE if the customer has been notified.'),
    '#callback' => 'uc_downpayment_condition_notified',
    '#arguments' => array(
      'payment' => $payment_arg,
    ),
  );

  return $conditions;
}

/**
 * Implementation of hook_ca_action().
 */
function uc_downpayment_ca_action() {
  $order_arg = array(
    '#entity' => 'uc_order',
    '#title' => t('Order'),
  );
  
 $payment_arg = array(
    '#entity' => 'uc_downpayment',
    '#title' => t('Down payment'),
  );
  
  $actions['uc_downpayment_action_update_status'] = array(
    '#title' => t('Update the down payment status'),
    '#category' => t('Down payment'),
    '#callback' => 'uc_downpayment_action_update_status',
    '#arguments' => array(
      'order' => $order_arg,
    ),
  );
  $actions['uc_downpayment_action_update_notified'] = array(
    '#title' => t('Update the notification status'),
    '#category' => t('Down payment'),
    '#callback' => 'uc_downpayment_action_update_notified',
    '#arguments' => array(
      'payment' => $payment_arg,
    ),
  );
  $actions['uc_downpayment_email'] = array(
    '#title' => t('Send an down payment email'),
    '#category' => t('Down payment'),
    '#callback' => 'uc_downpayment_action_email',
    '#arguments' => array(
      'payment' => $payment_arg,
    ),
  );

  return $actions;
}

function uc_downpayment_action_update_status($order, $settings) {
  foreach ($order->products as $product) {
    if ($product->data['downpayment']->dpid) {
      db_query("UPDATE {uc_downpayment_payments} SET status = %d WHERE dpid = %d", $settings['status'], $product->data['downpayment']->dpid);
    }
  }
}

function uc_downpayment_action_update_status_form($form_state, $settings = array()) {
  $form['status'] = array(
    '#type' => 'select',
    '#title' => t('New status'),
    '#options' => uc_downpayment_payment_status_options(),
    '#default_value' => $settings['status'],
  );

  return $form;
}

function uc_downpayment_action_update_notified($payment, $settings) {
  $payment->notified = time();
  uc_downpayment_save($payment);
}

function uc_downpayment_action_update_notified_form($form_state, $settings = array()) {
  $form = array();
  return $form;
}

function uc_downpayment_condition_notified($payment, $settings) {
  return $payment->notified == 0;
}

function uc_downpayment_condition_notified_form($form_state, $settings = array()) {
  $form = array();
  /*
  $form['notified'] = array(
    '#type' => 'checkbox',
    '#title' => t('Customer has been notified'),
    '#default_value' => $settings['notified'],
  );
*/
  return $form;
}

/**
 * Send an email concerning an down payment.
 */
function uc_downpayment_action_email($downpayment, $settings) {
  $order = uc_order_load($downpayment->order_id);
  $account = uc_order_user_load($order);
 //print_r($settings);
  // Token replacements for the subject and body
  $settings['replacements'] = array(
    'global' => NULL,
    'order' => $order,
    'user' => $account,
    'downpayment' => $downpayment,
  );
 
  // Apply token replacements to from and recipient e-mail addressses.
  $from = token_replace_multiple($settings['from'], $settings['replacements']);
  $addresses = token_replace_multiple($settings['addresses'], $settings['replacements']);

  // Split up our recipient e-mail addresses.
  $recipients = array();
  foreach (explode("\n", $addresses) as $address) {
    $recipients[] = trim($address);
  }
   
   //  exit;
  foreach ($recipients as $email) {
    $sent = drupal_mail('uc_downpayment', 'action-mail', $email, uc_store_mail_recipient_language($email), $settings, empty($settings['from']) ? uc_store_email_from() : $settings['from']);

    if (!$sent['result']) {
      watchdog('ca', 'Down payment due mail failed.', WATCHDOG_ERROR);
    }
  }
}

/**
 * @see uc_downpayment_action_email()
 */
function uc_downpayment_action_email_form($form_state, $settings = array()) {
  $form['from'] = array(
    '#type' => 'textfield',
    '#title' => t('Sender'),
    '#default_value' => isset($settings['from']) ? $settings['from'] : uc_store_email_from(),
    '#description' => t('The "From" address.'),
    '#required' => TRUE,
  );
  $form['addresses'] = array(
    '#type' => 'textarea',
    '#title' => t('Recipients'),
    '#default_value' => isset($settings['addresses']) ? $settings['addresses'] : '[order-email]',
    '#description' => t('Enter the email addresses to receive the notifications, one on each line. You may use order tokens for dynamic email addresses.'),
    '#required' => TRUE,
  );
  $form['subject'] = array(
    '#type' => 'textfield',
    '#title' => t('Subject'),
    '#default_value' => $settings['subject'],
    '#required' => TRUE,
  );
  $form['message'] = array(
    '#type' => 'textarea',
    '#title' => t('Message'),
    '#default_value' => $settings['message'],
  );

  // We add the #is_format element to allow us to locate and configure the filters.
  $form['format'] = filter_form($settings['format']) + array('#is_format' => TRUE);

  $form['token_help'] = array(
    '#type' => 'fieldset',
    '#title' => t('Replacement patterns'),
    '#description' => t('You can make use of the replacement patterns in the e-mail from and recipients fields, the subject, and the message body.'),
    '#collapsible' => TRUE,
    '#collapsed' => TRUE,
  );
  foreach (array('global', 'order', 'downpayment') as $name) {
    $form['token_help'][$name] = array(
      '#type' => 'fieldset',
      '#title' => t('@name replacement patterns', array('@name' => drupal_ucfirst($name))),
      '#collapsible' => TRUE,
      '#collapsed' => TRUE,
    );
    $form['token_help'][$name]['content'] = array(
      '#value' => theme('token_help', $name),
    );
  }

  return $form;
}
